class marketplaces:
    @staticmethod
    def US():
        # United State
        url = "https://sellingpartnerapi-na.amazon.com"
        marketplace_id = "ATVPDKIKX0DER"
        return url, marketplace_id

    @staticmethod
    def CA():
        # Canada
        url = "https://sellingpartnerapi-na.amazon.com"
        marketplace_id = "A2EUQ1WTGCTBG2"
        return url, marketplace_id

    @staticmethod
    def MX():
        # Mexico
        url = "https://sellingpartnerapi-na.amazon.com"
        marketplace_id = "A1AM78C64UM0Y8"
        return url, marketplace_id

    @staticmethod
    def BR():
        # Brazil
        url = "https://sellingpartnerapi-na.amazon.com"
        marketplace_id = "A2Q3Y263D00KWC"
        return url, marketplace_id

    @staticmethod
    def ES():
        # Spain
        url = "https://sellingpartnerapi-eu.amazon.com"
        marketplace_id = "A1RKKUPIHCS9HS"
        return url, marketplace_id

    @staticmethod
    def UK():
        # United Kingdom
        url = "https://sellingpartnerapi-eu.amazon.com"
        marketplace_id = "A1F83G8C2ARO7P"
        return url, marketplace_id

    @staticmethod
    def FR():
        # France
        url = "https://sellingpartnerapi-eu.amazon.com"
        marketplace_id = "A13V1IB3VIYZZH"
        return url, marketplace_id

    @staticmethod
    def BE():
        # Belgium
        url = "https://sellingpartnerapi-eu.amazon.com"
        marketplace_id = "AMEN7PMS3EDWL"
        return url, marketplace_id

    @staticmethod
    def NL():
        # Netherlands
        url = "https://sellingpartnerapi-eu.amazon.com"
        marketplace_id = "A1805IZSGTT6HS"
        return url, marketplace_id

    @staticmethod
    def DE():
        # Germany
        url = "https://sellingpartnerapi-eu.amazon.com"
        marketplace_id = "A1PA6795UKMFR9"
        return url, marketplace_id

    @staticmethod
    def IT():
        # Italy
        url = "https://sellingpartnerapi-eu.amazon.com"
        marketplace_id = "APJ6JRA9NG5V4"
        return url, marketplace_id

    @staticmethod
    def SE():
        # Sweden
        url = "https://sellingpartnerapi-eu.amazon.com"
        marketplace_id = "A2NODRKZP88ZB9"
        return url, marketplace_id

    @staticmethod
    def PL():
        # Poland
        url = "https://sellingpartnerapi-eu.amazon.com"
        marketplace_id = "A1C3SOZRARQ6R3"
        return url, marketplace_id

    @staticmethod
    def TR():
        # Turkey
        url = "https://sellingpartnerapi-eu.amazon.com"
        marketplace_id = "A33AVAJ2PDY3EV"
        return url, marketplace_id

    @staticmethod
    def ZA():
        # South Africa
        url = "https://sellingpartnerapi-eu.amazon.com"
        marketplace_id = "AE08WJ6YKNBMC"
        return url, marketplace_id

    @staticmethod
    def EG():
        # Egypt
        url = "https://sellingpartnerapi-eu.amazon.com"
        marketplace_id = "ARBP9OOSHTCHU"
        return url, marketplace_id

    @staticmethod
    def SA():
        # Saoudi Arabia
        url = "https://sellingpartnerapi-eu.amazon.com"
        marketplace_id = "A17E79C6D8DWNP"
        return url, marketplace_id

    @staticmethod
    def AE():
        # United Arab Emerates
        url = "https://sellingpartnerapi-eu.amazon.com"
        marketplace_id = "A2VIGQ35RCS4UG"
        return url, marketplace_id

    @staticmethod
    def IN():
        # India
        url = "https://sellingpartnerapi-eu.amazon.com"
        marketplace_id = "A21TJRUUN4KGV"
        return url, marketplace_id

    @staticmethod
    def JP():
        # Japan
        url = "https://sellingpartnerapi-fe.amazon.com"
        marketplace_id = "A1VC38T7YXB528"
        return url, marketplace_id

    @staticmethod
    def AU():
        # Australia
        url = "https://sellingpartnerapi-fe.amazon.com"
        marketplace_id = "A39IBJ37TRP1C6"
        return url, marketplace_id

    @staticmethod
    def SG():
        # Singapore
        url = "https://sellingpartnerapi-fe.amazon.com"
        marketplace_id = "A19VAU5U5O7RUS"
        return url, marketplace_id



import pandas as pd
import numpy as np
from datetime import datetime, timezone

def wmOrdersReport(filePath:str):
    ordersDf = pd.read_excel(filePath)
    ordersDf = ordersDf.rename(columns=lambda x:x.replace(' ','_').replace('-','_').replace('#','').lower())

    ordersDf['order_date'] = pd.to_datetime(ordersDf['order_date'])
    ordersDf['delivery_date'] = pd.to_datetime(ordersDf['delivery_date'])
    ordersDf['service_scheduled_date'] = pd.to_datetime(ordersDf['service_scheduled_date'])

    req_columns = [
        'po',
        'service_po',
        'order',
        'order_date',
        'ship_by',
        'delivery_date',
        'service_scheduled_date',
        'customer_name',
        'customer_shipping_address',
        'customer_phone_number',
        'ship_to_address_1',
        'ship_to_address_2',
        'ship_to_country',
        'city',
        'state',
        'zip',
        'segment',
        'flids',
        'line',
        'upc',
        'status',
        'service_status',
        'item_description',
        'shipping_method',
        'shipping_tier',
        'shipping_sla',
        'shipping_config_source',
        'qty',
        'sku',
        'condition',
        'item_cost',
        'total_service_cost',
        'discount',
        'shipping_cost',
        'tax',
        'carrier',
        'tracking_number',
        'tracking_url',
        'update_status',
        'update_qty',
        'update_carrier',
        'update_tracking_number',
        'update_tracking_url',
        'seller_order_no',
        'fulfillment_entity',
        'replacement_order',
        'original_customer_order_id',
        'customer_intent_to_cancel',
        'intent_to_cancel_override',
        'platform_type'
    ]

    for col in req_columns:
        if col not in ordersDf.columns:
            ordersDf[col] = np.nan

    ordersDf = ordersDf[req_columns]

    schema = {
        'po': str,
        'service_po': str,
        'order': str,
        'order_date': 'datetime64[ns]',
        'ship_by': str,
        'delivery_date': 'datetime64[ns]',
        'service_scheduled_date': 'datetime64[ns]',
        'customer_name': str,
        'customer_shipping_address': str,
        'customer_phone_number': str,
        'ship_to_address_1': str,
        'ship_to_address_2': str,
        'ship_to_country': str,
        'city': str,
        'state': str,
        'zip': str,
        'segment': str,
        'flids': str,
        'line': str,
        'upc': str,
        'status': str,
        'service_status': str,
        'item_description': str,
        'shipping_method': str,
        'shipping_tier': str,
        'shipping_sla': float,
        'shipping_config_source': str,
        'qty': float,
        'sku': str,
        'condition': str,
        'item_cost': float,
        'total_service_cost': float,
        'discount': float,
        'shipping_cost': float,
        'tax': float,
        'carrier': str,
        'tracking_number': str,
        'tracking_url': str,
        'update_status': str,
        'update_qty': float,
        'update_carrier': str,
        'update_tracking_number': str,
        'update_tracking_url': str,
        'seller_order_no': str,
        'fulfillment_entity': str,
        'replacement_order': str,
        'original_customer_order_id': str,
        'customer_intent_to_cancel': str,
        'intent_to_cancel_override': str,
        'platform_type': str
    }

    ordersDf = ordersDf.astype(schema)

    return ordersDf

def wmAdsReport(filePath:str):
    adsDf = pd.read_csv(filePath, skiprows=1)
    adsDf = adsDf.rename(columns=lambda x:x.replace(' ','_').replace('(','').replace(')','').replace('#','').replace('-','').replace('.','').lower())
    adsDf['date'] = pd.to_datetime(adsDf['date'])
    adsDf = adsDf.replace({'\$':'',',':'','null':np.nan}, regex=True)

    req_columns = [
        'date',
        'item_id',
        'impressions',
        'clicks',
        'ad_spend',
        'total_units_sold__14_day',
        'advertised_sku_units__14_day',
        'other_sku_units___14_day',
        'orders__14_day',
        'total_sales_rev__14_day',
        'advertised_sku_sales__14_day',
        'other_sku_sales__14_day'
    ]

    missingColumns = set(req_columns) - set(adsDf.columns)
    newColumns = set(adsDf.columns) - set(req_columns)

    if missingColumns or newColumns:
        message = (
        f"""
        missing columns: {', '.join(missingColumns)}
        new columns: {', '.join(newColumns)}
        """
        )

        raise ValueError(message)

    adsDf = adsDf[req_columns]

    schema = {
        'date': 'datetime64[ns]',
        'item_id': str,
        'impressions': float,
        'clicks': float,
        'ad_spend': float,
        'total_units_sold__14_day': float,
        'advertised_sku_units__14_day': float,
        'other_sku_units___14_day': float,
        'orders__14_day': float,
        'total_sales_rev__14_day': float,
        'advertised_sku_sales__14_day': float,
        'other_sku_sales__14_day': float
    }

    adsDf = adsDf.astype(schema)

    return adsDf

def wmSettlementReport(filePath:str):
    settlementDf = pd.read_csv(filePath, skiprows=3)
    settlementDf = settlementDf.rename(columns=lambda x:x.replace(' ','_').replace('#','').replace('.','_').replace('/','_').lower())

    req_columns = [
        'walmart_com_order_',
        'walmart_com_order_line_',
        'walmart_com_po_',
        'walmart_com_po_line_',
        'transaction_type',
        'reason_code',
        'detail',
        'wfsreferenceid',
        'transaction_date_time',
        'qty',
        'partner_id',
        'partner_gtin',
        'partner_item_name',
        'category',
        'sub_category',
        'net_payable',
        'customerorderid',
        'orderchannelid'
    ]

    missingColumns = set(req_columns) - set(settlementDf.columns)
    newColumns = set(settlementDf.columns) - set(req_columns)

    if missingColumns or newColumns:
        message = (
        f"""
        missing columns: {', '.join(missingColumns)}
        new columns: {', '.join(newColumns)}
        """
        )

        raise ValueError(message)

    settlementDf = settlementDf[req_columns]
    settlementDf['transaction_date_time'] = pd.to_datetime(settlementDf['transaction_date_time'])

    schema = {
        'walmart_com_order_': str,
        'walmart_com_order_line_': float,
        'walmart_com_po_': str,
        'walmart_com_po_line_': float,
        'transaction_type': str,
        'reason_code': str,
        'detail': str,
        'wfsreferenceid': str,
        'transaction_date_time': 'datetime64[ns]',
        'qty': float,
        'partner_id': str,
        'partner_gtin': str,
        'partner_item_name': str,
        'category': str,
        'sub_category': str,
        'net_payable': float,
        'customerorderid': str,
        'orderchannelid': str
    }

    settlementDf = settlementDf.astype(schema)

    return settlementDf

def wmStorageReport(filePath:str):
    monthDf = pd.read_csv(filePath, skiprows=2)
    month = monthDf.columns[0]

    storageDf = pd.read_csv(filePath, skiprows=3)
    storageDf.insert(0, 'month', month)
    storageDf['month'] = pd.to_datetime(storageDf['month'])

    storageDf = storageDf.rename(columns=lambda x:x.replace(' ','_').replace('(','').replace(')','').replace('-','_').replace(',','').lower())

    req_columns = [
        'month',
        'partner_gtin',
        'vendor_sku',
        'walmart_item_id',
        'item_name',
        'length',
        'width',
        'height',
        'volume',
        'weight',
        'standard_daily_storage_cost_per_unit_off_peak_aged_under_365_days',
        'peak_daily_storage_cost_per_unit_aged_over_30_days',
        'long_term_daily_storage_cost_per_unit_aged_over_365_days',
        'average_units_on_hand',
        'ending_units_on_hand',
        'storage_fee_for_selected_time_period'
    ]

    missingColumns = set(req_columns) - set(storageDf.columns)
    newColumns = set(storageDf.columns) - set(req_columns)

    if missingColumns or newColumns:
        message = (
        f"""
        missing columns: {', '.join(missingColumns)}
        new columns: {', '.join(newColumns)}
        """
        )

        raise ValueError(message)

    schema = {
        'month': 'datetime64[ns]',
        'partner_gtin': str,
        'vendor_sku': str,
        'walmart_item_id': str,
        'item_name': str,
        'length': float,
        'width': float,
        'height': float,
        'volume': float,
        'weight': float,
        'standard_daily_storage_cost_per_unit_off_peak_aged_under_365_days': float,
        'peak_daily_storage_cost_per_unit_aged_over_30_days': float,
        'long_term_daily_storage_cost_per_unit_aged_over_365_days': float,
        'average_units_on_hand': float,
        'ending_units_on_hand': float,
        'storage_fee_for_selected_time_period': float
    }

    storageDf = storageDf.astype(schema)

    return storageDf




import requests
import pandas as pd
import numpy as np
from datetime import datetime, timedelta, timezone
import time
from .ratelimit import RateLimiter
from .fcmap import fc_to_country
from .marketplaces import marketplaces
import gzip
import shutil
import os
import urllib.parse

def shipmentEvents_daterange(marketplace_action, access_token, start_date, end_date):
    """
    This will pull Shipment Event Reports per region in a specific date range.

    Parameter:
    - marketplace_action: the specific marketplace command to pull the data
    - access_token: matching access token of the marketplace
    - start_date: start date in ISO format, this is inclusive
    - end_date: end date in ISO format, this is inclusive

    return:
    - data frame of Shipment Event Report
    """
    # Pull API Data
    rate_limiter = RateLimiter(tokens_per_second=0.5, capacity=30)
    records = []
    regionUrl, marketplace_id = marketplace_action()
    NextToken = None

    headers = {
            'x-amz-access-token': access_token
        }

    request_params  = {
        'PostedAfter': start_date,
        'PostedBefore': end_date
    }

    try:
        url = regionUrl + f'/finances/v0/financialEvents' + '?' + urllib.parse.urlencode(request_params)
        response = requests.get(url, headers=headers)
        records.extend(response.json()['payload']['FinancialEvents']['ShipmentEventList'])

        try:
            NextToken = response.json()['payload']['NextToken']
        except:
            NextToken = None

        while NextToken:
            request_params_next  = {
                'NextToken': NextToken
            }
            url = regionUrl + f'/finances/v0/financialEvents' + '?' + urllib.parse.urlencode(request_params_next)
            response = rate_limiter.send_request(requests.get, url, headers=headers)
            records.extend(response.json()['payload']['FinancialEvents']['ShipmentEventList'])

            try:
                NextToken = response.json()['payload']['NextToken']
            except:
                NextToken = None
            
        print('End of List')

    except Exception as e:
        print(response.json()['errors'][0]['message'])
        print(response.json()['errors'][0]['details'])

    # set Data Frame
    taxDf = []
    for record in records:
        data ={
            'amazon_order_id': record.get('AmazonOrderId', np.nan),
            'posted_date': record.get('PostedDate', np.nan),
            'marketplace': record.get('MarketplaceName', np.nan),
            'sku': record.get('ShipmentItemList', [{}])[0].get('SellerSKU', np.nan),
            'qty': record.get('ShipmentItemList', [{}])[0].get('QuantityShipped', np.nan),
            'currency': record.get('ShipmentItemList', [{}])[0].get('ItemChargeList', [{}])[0].get('ChargeAmount',{}).get('CurrencyCode', np.nan),
        }

        charges = record.get('ShipmentItemList', [{}])[0].get('ItemChargeList', [])
        for charge in charges:
            data[charge.get('ChargeType')] = charge.get('ChargeAmount', {}).get('CurrencyAmount', np.nan)

        fees = record.get('ShipmentItemList', [{}])[0].get('ItemFeeList', [])
        for fee in fees:
            data[fee.get('FeeType')] = fee.get('FeeAmount', {}).get('CurrencyAmount', np.nan)

        withhelds = record.get('ShipmentItemList', [{}])[0].get('ItemTaxWithheldList', [{}])[0].get('TaxesWithheld',[])
        for withheld in withhelds:
            data[withheld.get('ChargeType')] = withheld.get('ChargeAmount', {}).get('CurrencyAmount', np.nan)

        taxDf.append(data)

    taxDf = pd.DataFrame(taxDf)

    taxDf['posted_date'] = pd.to_datetime(taxDf['posted_date'])

    req_columns = [
        'amazon_order_id',
        'posted_date',
        'marketplace',
        'sku',
        'qty',
        'currency',
        'Principal',
        'Tax',
        'GiftWrap',
        'GiftWrapTax',
        'ShippingCharge',
        'ShippingTax',
        'FBAPerUnitFulfillmentFee',
        'Commission',
        'FixedClosingFee',
        'GiftwrapChargeback',
        'SalesTaxCollectionFee',
        'ShippingChargeback',
        'VariableClosingFee',
        'DigitalServicesFee',
        'FBAPerOrderFulfillmentFee',
        'FBAWeightBasedFee',
        'MarketplaceFacilitatorTax-Principal',
        'MarketplaceFacilitatorTax-Shipping',
        'MarketplaceFacilitatorVAT-Principal',
        'LowValueGoodsTax-Shipping',
        'LowValueGoodsTax-Principal',
        'MarketplaceFacilitatorVAT-Shipping',
        'MarketplaceFacilitatorTax-Other',
        'RenewedProgramFee'
    ]

    for col in req_columns:
        if col not in taxDf.columns:
            taxDf[col] = np.nan

    taxDf = taxDf[req_columns]

    schema = {
        'amazon_order_id': str,
        'posted_date': 'datetime64[ns, UTC]',
        'marketplace': str,
        'sku': str,
        'qty': float,
        'currency': str,
        'Principal': float,
        'Tax': float,
        'GiftWrap': float,
        'GiftWrapTax': float,
        'ShippingCharge': float,
        'ShippingTax': float,
        'FBAPerUnitFulfillmentFee': float,
        'Commission': float,
        'FixedClosingFee': float,
        'GiftwrapChargeback': float,
        'SalesTaxCollectionFee': float,
        'ShippingChargeback': float,
        'VariableClosingFee': float,
        'DigitalServicesFee': float,
        'FBAPerOrderFulfillmentFee': float,
        'FBAWeightBasedFee': float,
        'MarketplaceFacilitatorTax-Principal': float,
        'MarketplaceFacilitatorTax-Shipping': float,
        'MarketplaceFacilitatorVAT-Principal': float,
        'LowValueGoodsTax-Shipping': float,
        'LowValueGoodsTax-Principal': float,
        'MarketplaceFacilitatorVAT-Shipping': float,
        'MarketplaceFacilitatorTax-Other': float,
        'RenewedProgramFee': float
    }

    taxDf = taxDf.astype(schema)

    return taxDf

def refunds_daterange(marketplace_action, access_token, start_date, end_date):
    """
    This will pull Refund Reports per region in a specific date range.

    Parameter:
    - marketplace_action: the specific marketplace command to pull the data
    - access_token: matching access token of the marketplace
    - start_date: start date in ISO format, this is inclusive
    - end_date: end date in ISO format, this is inclusive

    return:
    - data frame of Refund Report
    """
    # Pull API Data
    rate_limiter = RateLimiter(tokens_per_second=0.5, capacity=30)
    records = []
    regionUrl, marketplace_id = marketplace_action()
    NextToken = None

    headers = {
            'x-amz-access-token': access_token
        }

    request_params  = {
        'PostedAfter': start_date,
        'PostedBefore': end_date
    }

    try:
        url = regionUrl + f'/finances/v0/financialEvents' + '?' + urllib.parse.urlencode(request_params)
        response = requests.get(url, headers=headers)
        records.extend(response.json()['payload']['FinancialEvents']['RefundEventList'])

        try:
            NextToken = response.json()['payload']['NextToken']
        except:
            NextToken = None

        while NextToken:
            request_params_next  = {
                'NextToken': NextToken
            }
            url = regionUrl + f'/finances/v0/financialEvents' + '?' + urllib.parse.urlencode(request_params_next)
            response = rate_limiter.send_request(requests.get, url, headers=headers)
            records.extend(response.json()['payload']['FinancialEvents']['RefundEventList'])

            try:
                NextToken = response.json()['payload']['NextToken']
            except:
                NextToken = None
            
        print('End of List')

    except Exception as e:
        print(response.json()['errors'][0]['message'])
        print(response.json()['errors'][0]['details'])

    # Data Frame
    refunds = []
    for record in records:
        data = {
            'amazon_order_id': record.get('AmazonOrderId', np.nan),
            'seller_order_id': record.get('SellerOrderId', np.nan),
            'marketplace_link': record.get('MarketplaceName', np.nan),
            'refund_date': record.get('PostedDate', np.nan),
            'sku': record.get('ShipmentItemAdjustmentList', [{}])[0].get('SellerSKU', np.nan),
            'order_adjustment_item_id': record.get('ShipmentItemAdjustmentList', [{}])[0].get('OrderAdjustmentItemId', np.nan),
            'quantity': record.get('ShipmentItemAdjustmentList', [{}])[0].get('QuantityShipped', np.nan),
            'currency': record.get('ShipmentItemAdjustmentList', [{}])[0].get('ItemChargeAdjustmentList', [{}])[0].get('ChargeAmount',{}).get('CurrencyCode',np.nan),
        }

        chargeAdjustment = record.get('ShipmentItemAdjustmentList', [{}])[0].get('ItemChargeAdjustmentList', [])
        for charge in chargeAdjustment:
            data['charge_' + charge.get('ChargeType')] = charge.get('ChargeAmount', {}).get('CurrencyAmount', np.nan)

        feeAdjustment = record.get('ShipmentItemAdjustmentList', [{}])[0].get('ItemFeeAdjustmentList', [])
        for fee in feeAdjustment:
            data['fee_' + fee.get('FeeType')] = fee.get('FeeAmount', {}).get('CurrencyAmount', np.nan)

        taxWithheld = record.get('ShipmentItemAdjustmentList', [{}])[0].get('ItemTaxWithheldList', [{}])[0].get('TaxesWithheld',[])
        for tax in taxWithheld:
            data['tax_' + tax.get('ChargeType')] = tax.get('ChargeAmount', {}).get('CurrencyAmount', np.nan)

        refunds.append(data)

    refundsDf = pd.DataFrame(refunds)
    refundsDf = refundsDf.rename(columns=lambda x:x.replace('-','_').lower())

    req_columns = [
        'amazon_order_id',
        'seller_order_id',
        'marketplace_link',
        'refund_date',
        'sku',
        'order_adjustment_item_id',
        'quantity',
        'currency',
        'charge_tax',
        'charge_principal',
        'fee_commission',
        'fee_fixedclosingfee',
        'fee_giftwrapchargeback',
        'fee_refundcommission',
        'fee_shippingchargeback',
        'fee_variableclosingfee',
        'fee_digitalservicesfee',
        'tax_marketplacefacilitatortax_principal',
        'charge_restockingfee',
        'charge_shippingtax',
        'charge_shippingcharge',
        'fee_salestaxcollectionfee',
        'charge_returnshipping',
        'tax_marketplacefacilitatortax_shipping',
        'charge_exportcharge',
        'charge_giftwrap',
        'charge_giftwraptax',
        'tax_marketplacefacilitatorvat_principal',
        'fee_renewedprogramfee',
        'tax_marketplacefacilitatorvat_shipping'
    ]

    for col in req_columns:
        if col not in refundsDf.columns:
            refundsDf[col] = np.nan

    refundsDf = refundsDf[req_columns]

    schema = {
        'amazon_order_id': str,
        'seller_order_id': str,
        'marketplace_link': str,
        'refund_date': 'datetime64[ns, UTC]',
        'sku': str,
        'order_adjustment_item_id': str,
        'quantity': float,
        'currency': str,
        'charge_tax': float,
        'charge_principal': float,
        'fee_commission': float,
        'fee_fixedclosingfee': float,
        'fee_giftwrapchargeback': float,
        'fee_refundcommission': float,
        'fee_shippingchargeback': float,
        'fee_variableclosingfee': float,
        'fee_digitalservicesfee': float,
        'tax_marketplacefacilitatortax_principal': float,
        'charge_restockingfee': float,
        'charge_shippingtax': float,
        'charge_shippingcharge': float,
        'fee_salestaxcollectionfee': float,
        'charge_returnshipping': float,
        'tax_marketplacefacilitatortax_shipping': float,
        'charge_exportcharge': float,
        'charge_giftwrap': float,
        'charge_giftwraptax': float,
        'tax_marketplacefacilitatorvat_principal': float,
        'fee_renewedprogramfee': float,
        'tax_marketplacefacilitatorvat_shipping': float
    }

    refundsDf = refundsDf.astype(schema)

    return refundsDf

def all_orders_daterange(marketplace_action, access_token, start_date, end_date, file_path_name):
    """
    This will pull All Orders Report per region in a specific date range.

    Parameter:
    - marketplace_action: the specific marketplace command to pull the data
    - access_token: matching access token of the marketplace
    - start_date: start date in ISO format, this is inclusive
    - end_date: end date in ISO format, this is inclusive
    - file_path_name: file path up to file name where to save the initially downloaded report

    return:
    - data frame of Refund Report
    """
    # Create Report
    regionUrl , marketplace_id = marketplace_action()
    endpoint = f'/reports/2021-06-30/reports'
    url = regionUrl + endpoint
    headers = {
        'x-amz-access-token': access_token,
        'Content-Type': 'application/json'
    }

    request_params = {
        'marketplaceIds': [marketplace_id],
        'reportType': 'GET_FLAT_FILE_ALL_ORDERS_DATA_BY_ORDER_DATE_GENERAL',
        'dataStartTime': start_date,
        'dataEndTime': end_date
    }

    create_response = requests.post(url, headers=headers, json=request_params,)
    report_id = create_response.json()['reportId']

    # Check Report Status
    endpoint = f'/reports/2021-06-30/reports/{report_id}'
    url = regionUrl + endpoint

    while True:
        status_response = requests.get(url, headers=headers)
        status = status_response.json().get("processingStatus")

        if status == "DONE":
            print("Report is ready for download!")
            document_id = status_response.json()["reportDocumentId"]
            break
        elif status == "CANCELLED":
            raise ValueError("Report creation was cancelled.")
        elif status == "FATAL":
            raise ValueError("Report creation failed.")
        else:
            print(f"Report status: {status}. Waiting for report to be ready...")
            time.sleep(60)  # Wait before checking again

    # Download Report
    endpoint = f'/reports/2021-06-30/documents/{document_id}'
    url = regionUrl + endpoint
    document_response = requests.get(url, headers=headers)

    if document_response.status_code == 200:
        download_url = document_response.json()["url"]
        compression_algorithm = document_response.json().get("compressionAlgorithm")
        report_data = requests.get(download_url)
        
        if compression_algorithm == "GZIP":
            print('Decompressing...')

            with open(file_path_name + '.gz', 'wb') as f:
                f.write(report_data.content)

            with gzip.open(file_path_name + '.gz', 'rb') as f_in:
                with open(file_path_name, 'wb') as f_out:
                    shutil.copyfileobj(f_in, f_out)

        else:
            with open(file_path_name, 'wb') as f:
                f.write(report_data.content)

        print('Report Downloaded!')

    else:
        print("Failed to get the report document:", document_response.json())

    # Process DF
    allOrdersDf = pd.read_csv(filepath_or_buffer=file_path_name, delimiter='\t')
    allOrdersDf = allOrdersDf.rename(columns=lambda x:x.replace('-','_').replace(' ','_').lower())
    allOrdersDf['purchase_date'] = pd.to_datetime(allOrdersDf['purchase_date'])
    allOrdersDf['last_updated_date'] = pd.to_datetime(allOrdersDf['last_updated_date'])

    req_columns = [
        'amazon_order_id',
        'merchant_order_id',
        'purchase_date',
        'last_updated_date',
        'order_status',
        'fulfillment_channel',
        'sales_channel',
        'order_channel',
        'ship_service_level',
        'product_name',
        'sku',
        'asin',
        'number_of_items',
        'item_status',
        'tax_collection_model',
        'tax_collection_responsible_party',
        'quantity',
        'currency',
        'item_price',
        'item_tax',
        'shipping_price',
        'shipping_tax',
        'gift_wrap_price',
        'gift_wrap_tax',
        'item_promotion_discount',
        'ship_promotion_discount',
        'address_type',
        'ship_city',
        'ship_state',
        'ship_postal_code',
        'ship_country',
        'promotion_ids',
        'payment_method_details',
        'cpf',
        'item_extensions_data',
        'is_business_order',
        'purchase_order_number',
        'price_designation',
        'buyer_company_name',
        'is_replacement_order',
        'is_exchange_order',
        'original_order_id',
        'license_state',
        'license_expiration_date',
        'is_buyer_requested_cancellation',
        'buyer_requested_cancel_reason',
        'is_transparency',
        'ioss_number',
        'signature_confirmation_recommended'
    ]

    for col in req_columns:
        if col not in allOrdersDf.columns:
            allOrdersDf[col] = np.nan

    allOrdersDf = allOrdersDf[req_columns]

    schema = {
        'amazon_order_id': str,
        'merchant_order_id': str,
        'purchase_date': 'datetime64[ns, UTC]',
        'last_updated_date': 'datetime64[ns, UTC]',
        'order_status': str,
        'fulfillment_channel': str,
        'sales_channel': str,
        'order_channel': str,
        'ship_service_level': str,
        'product_name': str,
        'sku': str,
        'asin': str,
        'number_of_items': float,
        'item_status': str,
        'tax_collection_model': str,
        'tax_collection_responsible_party': str,
        'quantity': float,
        'currency': str,
        'item_price': float,
        'item_tax': float,
        'shipping_price': float,
        'shipping_tax': float,
        'gift_wrap_price': float,
        'gift_wrap_tax': float,
        'item_promotion_discount': float,
        'ship_promotion_discount': float,
        'address_type': str,
        'ship_city': str,
        'ship_state': str,
        'ship_postal_code': str,
        'ship_country': str,
        'promotion_ids': str,
        'payment_method_details': str,
        'cpf': str,
        'item_extensions_data': str,
        'is_business_order': bool,
        'purchase_order_number': str,
        'price_designation': str,
        'buyer_company_name': str,
        'is_replacement_order': bool,
        'is_exchange_order': bool,
        'original_order_id': str,
        'license_state': str,
        'license_expiration_date': str,
        'is_buyer_requested_cancellation': bool,
        'buyer_requested_cancel_reason': str,
        'is_transparency': bool,
        'ioss_number': str,
        'signature_confirmation_recommended': bool
    }

    allOrdersDf = allOrdersDf.astype(schema)

    return allOrdersDf




from .reports import lowfeereport
from .reports import dfbgcolcheck
from .reports import bgdeldup
from .reports import bgdeldupf
from .reports import promoreport
from .reports import spstreport
from .reports import sbstreport
from .reports import sdtreport
from .reports import spcreport
from .reports import sbcreport
from .reports import sdcreport
from .reports import unifiedRepo
from .reports import storageFeeReport
from .reports import overAgeFeeReport
from .reports import returnProcessFeeReport
from .reports import fbaFeeReport
from .reports import inboundPlacementReport
from .reports import campAttribReport
from .reports import invWarehouseLoc
from .reports import spAdvertisedReport
from .reports import sdAdvertisedReport
from .reports import spTargetingReport
from .reports import shipmentsReport
from .reports import reimbursementReport
from .reports import ledgerReport
from .reports import replacementReport
from .reports import customerReturnReport
from .reports import spPlacementReport
from .reports import bgdeldupM
from .reports import spBulkReport
from .reports import sbBulkReport
from .reports import sdBulkReport
from .reports import bgdeldupfM

from .ratelimit import RateLimiter

from .marketplaces import marketplaces

from .api import zv_client_access
from .api import shipment_status
from .api import shipment_items
from .api import shipment_summary
from .api import narf_eligibility
from .api import fba_inventory
from .api import all_orders
from .api import reimbursement_report
from .api import inv_ledger
from .api import customer_return
from .api import replacements
from .api import offers
from .api import ss_forecast
from .api import ss_performance
from .api import finance_shipmentEventList
from .api import fba_inv_live
from .api import sku_attributes
from .api import get_sellerId
from .api import finance_refunds
from .api import fee_preview
from .api import manage_fba
from .api import awd_inventory
from .api import bsr
from .api import browse_tree
from .api import all_listing_report

from .spapi_daterange_reports import shipmentEvents_daterange
from .spapi_daterange_reports import refunds_daterange
from .spapi_daterange_reports import all_orders_daterange

from .walmart_report import wmOrdersReport
from .walmart_report import wmAdsReport
from .walmart_report import wmSettlementReport
from .walmart_report import wmStorageReport